// Copyright (C) 2005-2007 - John Burkardt
// Copyright (C) 2009-2010 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license
// http://www.gnu.org/copyleft/lesser.html

# include <cstdlib>
# include <cmath>
# include <ctime>
# include <iostream>
# include <sstream>
# include <fstream>
# include <iomanip>

using namespace std;

#include "halton.h"
#include "lowdisc_shared.h"

//
//  Reference:
//
//    J H Halton,
//    On the efficiency of certain quasi-random sequences of points
//    in evaluating multi-dimensional integrals,
//    Numerische Mathematik,
//    Volume 2, 1960, pages 84-90.
//
//    J H Halton and G B Smith,
//    Algorithm 247: Radical-Inverse Quasi-Random Point Sequence,
//    Communications of the ACM,
//    Volume 7, 1964, pages 701-702.
//
//    Ladislav Kocis and William Whiten,
//    Computational Investigations of Low-Discrepancy Sequences,
//    ACM Transactions on Mathematical Software,
//    Volume 23, Number 2, 1997, pages 266-294.
//
//  Author:
//
//    2004 - John Burkardt
//    2010 - Michael Baudin
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//

//
//  These variables are accessible to the user via calls to set/get routines.
//
int  halton_DIM_NUM = -1;
int *halton_BASE = NULL;
int *halton_LEAP = NULL;
int *halton_SEED = NULL;
bool halton_startup = false;

//****************************************************************************80

void halton ( int step , double r[] )

//****************************************************************************80
//
//  Purpose:
//
//    HALTON computes the next element in a leaped Halton subsequence.
//
//  Discussion:
//
//    The DIM_NUM-dimensional Halton sequence is really DIM_NUM separate
//    sequences, each generated by a particular base.
//
//    This routine selects elements of a "leaped" subsequence of the 
//    Halton sequence.  The subsequence elements are indexed by a
//    quantity called STEP, which starts at 0.  The STEP-th subsequence 
//    element is simply element 
//
//      SEED(1:DIM_NUM) + STEP * LEAP(1:DIM_NUM) 
//
//    of the original Halton sequence.
//
//
//    This routine "hides" a number of input arguments.  To specify these
//    arguments explicitly, use I4_TO_HALTON instead.
//
//    All the arguments have default values.  However, if you want to
//    examine or change them, you may call the appropriate routine
//    before calling HALTON.
//
//    The arguments that the user may set include:
//
//    * DIM_NUM, the spatial dimension, 
//      Default: DIM_NUM = 1;
//      Required: 1 <= DIM_NUM is required.
//
//    * STEP, the subsequence index.
//      Default: STEP = 0.
//      Required: 0 <= STEP.
//
//    * SEED(1:DIM_NUM), the Halton sequence element corresponding to STEP = 0.
//      Default SEED = (0, 0, ... 0).  
//      Required: 0 <= SEED(1:DIM_NUM).
//
//    * LEAP(1:DIM_NUM), the succesive jumps in the Halton sequence.
//      Default: LEAP = (1, 1, ..., 1). 
//      Required: 1 <= LEAP(1:DIM_NUM).
//
//    * BASE(1:DIM_NUM), the Halton bases.
//      Default: BASE = (2, 3, 5, 7, 11, ... ). 
//      Required: 1 < BASE(1:DIM_NUM).
//
//  Parameters:
//
//    Output, double R[DIM_NUM], the next element of the leaped Halton
//    subsequence.
//
{
	int i;
	int seed2;
	double base_inv;
	int digit;

	if ( !halton_startup )
	{
		ostringstream msg;
		msg << "halton - halton - Error!\n";
		msg << "  Startup is not done.\n";
		lowdisc_error(msg.str());
		return;
	}

	for ( i = 0; i < halton_DIM_NUM; i++ )
	{
		seed2 = halton_SEED[i] + step * halton_LEAP[i];

		r[i] = 0.0;

		base_inv = 1.0 / ( ( double ) halton_BASE[i] );

		while ( seed2 != 0 )
		{
			digit = seed2 % halton_BASE[i];
			r[i] = r[i] + ( ( double ) digit ) * base_inv;
			base_inv = base_inv / ( ( double ) halton_BASE[i] );
			seed2 = seed2 / halton_BASE[i];
		}
	}

	return;
}

//****************************************************************************80


int halton_dim_num_get ( void )
//    HALTON_DIM_NUM_GET gets the spatial dimension for a leaped Halton subsequence.
//  Parameters:
//    Output, int HALTON_DIM_NUM_GET, the spatial dimension.
{
	return halton_DIM_NUM;
}
//****************************************************************************80

void halton_base_get ( int base[] )
//    HALTON_BASE_GET gets the base vector for a leaped Halton subsequence.
//  Parameters:
//    Output, int base[], the bases
{
	int i;
	for ( i = 0; i < halton_DIM_NUM; i++ )
	{
		base[i] = halton_BASE[i];
	}
	return;
}
//****************************************************************************80


void halton_leap_get ( int leap[] )
//    HALTON_LEAP_GET gets the leap vector for a leaped Halton subsequence.
//  Parameters:
//    Output, int leap[], the successive jumps in the Halton sequence.
{
	int i;
	for ( i = 0; i < halton_DIM_NUM; i++ )
	{
		leap[i] = halton_LEAP[i];
	}
	return;
}
//****************************************************************************80

void halton_seed_get ( int seed[] )

//    HALTON_SEED_GET gets the seed vector for a leaped Halton subsequence.
//  Parameters:
//    Output, int seed[], the sequence index corresponding to STEP = 0.
{
	int i;
	for ( i = 0; i < halton_DIM_NUM; i++ )
	{
		seed[i] = halton_SEED[i];
	}
	return;
}
//****************************************************************************80

void halton_start ( int dim_num , int base[] , int seed[] , int leap[] )

//    halton_start startup the sequence.
//	
//  Parameters:
//
//    Input, int faure_dim_num, the spatial dimension, which should be
//    at least 2.
//
//    * DIM_NUM, the spatial dimension, 
//      Default: DIM_NUM = 1;
//      Required: 1 <= DIM_NUM is required.
//
//    * STEP, the subsequence index.
//      Default: STEP = 0.
//      Required: 0 <= STEP.
//
//    * SEED(1:DIM_NUM), the Halton sequence element corresponding to STEP = 0.
//      Default SEED = (0, 0, ... 0).  
//      Required: 0 <= SEED(1:DIM_NUM).
//
//    * LEAP(1:DIM_NUM), the succesive jumps in the Halton sequence.
//      Default: LEAP = (1, 1, ..., 1). 
//      Required: 1 <= LEAP(1:DIM_NUM).
//
//    * BASE(1:DIM_NUM), the Halton bases.
//      Default: BASE = (2, 3, 5, 7, 11, ... ). 
//      If BASE[i] = 0 then the prime number #i is automatically set.
//	    If BASE[i] > 1 then the base is directly used.
//	    If BASE[i] <0 or BASE[i] = 1, this is an error.

{
	int i;

	if ( halton_startup )
	{
		ostringstream msg;
		msg << "halton - halton_start - Error!\n";
		msg << "  Startup is already done.\n";
		lowdisc_error(msg.str());
		return;
	}
	halton_startup = true;
	//
	// Store the dimension
	//
	if ( dim_num < 1 )
	{
		ostringstream msg;
		msg << "halton - halton_start - Error" << endl;
		msg << "  The spatial dimension DIM_NUM is lower than 1." << endl;
		msg << "  But this input value is DIM_NUM = " << dim_num << endl;
		lowdisc_error(msg.str());
		return;
	}
	halton_DIM_NUM = dim_num;
	//
	// Store the seed
	//
	halton_SEED = new int[halton_DIM_NUM];
	for ( i = 0; i < halton_DIM_NUM; i++ )
	{
		if ( seed[i] < 0 ) 
		{
			ostringstream msg;
			msg << "halton - halton_start - Error!\n";
			msg << "  SEED entries must be nonnegative.\n";
			msg << "  seed[" << i << "] = " << seed[i] << "\n";
			lowdisc_error ( msg.str() );
			return;
		}
		halton_SEED[i] = seed[i];
	}
	//
	// Set the leap
	//
	halton_LEAP = new int[halton_DIM_NUM];
	for ( i = 0; i < halton_DIM_NUM; i++ )
	{
		if ( leap[i] < 1 ) 
		{
			ostringstream msg;
			msg << "halton - halton_start - Error!\n";
			msg << "  Leap entries must be greater than 0.\n";
			msg << "  leap[" << i << "] = " << leap[i] << "\n";
			lowdisc_error ( msg.str() );
			return;
		}
		halton_LEAP[i] = leap[1];
	}
	//
	// Set the base
	//
	halton_BASE = new int[halton_DIM_NUM];
	for ( i = 0; i < halton_DIM_NUM; i++ )
	{
		if ( base[i] < 0 || base[i] == 1 ) 
		{
			ostringstream msg;
			msg << "halton - halton_start - Error!\n";
			msg << "  Bases must be greater than 1.\n";
			msg << "  base[" << i << "] = " << base[i] << "\n";
			lowdisc_error ( msg.str() );
			return;
		}
		else if ( base[i] == 0 ) 
		{
			halton_BASE[i] = prime ( i + 1 );
		} 
		else
		{
			halton_BASE[i] = base[i];
		}
	}

	return;
}

//****************************************************************************80

void halton_stop ( )

//****************************************************************************80
//
//  Purpose:
//
//    halton_stop shutdown the sequence.
//	  Deletes the unnecessary memory.
//	
//  Parameters:
//
{
	if ( !halton_startup )
	{
		ostringstream msg;
		msg << "halton - halton_stop - Error!\n";
		msg << "  Startup is not done.\n";
		lowdisc_error(msg.str());
		return;
	}
	halton_startup = false;

	delete [] halton_BASE;
	delete [] halton_LEAP;
	delete [] halton_SEED;

	return;
}
//***************************************************************************
//  halton_isstart --
//     Returns true if the sequence is already started up.
//
//  Parameters:
//    startup, output : true if the sequence is already started up.
//
bool halton_isstart ( )
{
	return halton_startup;
}
