// Copyright (C) 2005-2007 - John Burkardt
// Copyright (C) 2009 - INRIA - Michael Baudin
// Copyright (C) 2009-2010 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license
// http://www.gnu.org/copyleft/lesser.html

#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <ctime>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <sstream>
#include <fstream>
#include <cstring>

using namespace std;

#include "lowdisc.h"
#include "lowdisc_shared.h"

// If this function is non-NULL, it is used to when an error is generated.
// It can be configure with the public function "lowdisc_errorsetfunction".
void (* lowdisc_errorfunction)(char * message) = NULL;
// If this function is non-NULL, it is used to print messages out.
// It can be configure with the public function "lowdisc_msgsetfunction".
void (* lowdisc_messagefunction)(char * message) = NULL;

//****************************************************************************80
// Prints out a message
void lowdisc_message ( char * message ) {
	if (lowdisc_messagefunction==NULL) {
		cout << message;
	} 
	else 
	{
		lowdisc_messagefunction(message);
	}
}

void lowdisc_message ( const string & str ) {
	char * cstr;
	cstr = new char [str.size()+1];
	strcpy (cstr, str.c_str());
	lowdisc_message ( cstr );
	delete [] cstr;
}

//****************************************************************************80
// Generates an error.
void lowdisc_error ( char * message ) {
	lowdisc_message ( "Low Discrepancy Module Error !" );
	if (lowdisc_errorfunction==NULL) {
		lowdisc_message ( message );
		exit(1);
	} 
	else 
	{
		lowdisc_errorfunction ( message );
	}
}
void lowdisc_error ( const string & str ) {
	char * cstr;
	cstr = new char [str.size()+1];
	strcpy (cstr, str.c_str());
	lowdisc_error ( cstr );
	delete [] cstr;
}


//****************************************************************************80

int i4_max ( int i1, int i2 )

//****************************************************************************80
//
//  Purpose:
//
//    I4_MAX returns the maximum of two I4's.
//
//  Modified:
//
//    13 October 1998
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, int I1, I2, are two integers to be compared.
//
//    Output, int I4_MAX, the larger of I1 and I2.
//
//
{
	if ( i2 < i1 )
	{
		return i1;
	}
	else
	{
		return i2;
	}

}
//****************************************************************************80

int i4_min ( int i1, int i2 )

//****************************************************************************80
//
//  Purpose:
//
//    I4_MIN returns the smaller of two I4's.
//
//  Modified:
//
//    13 October 1998
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, int I1, I2, two integers to be compared.
//
//    Output, int I4_MIN, the smaller of I1 and I2.
//
//
{
	if ( i1 < i2 )
	{
		return i1;
	}
	else
	{
		return i2;
	}

}
//****************************************************************************80

int i4_power ( int i, int j )

//****************************************************************************80
//
//  Purpose:
//
//    I4_POWER returns the value of I^J.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    01 April 2004
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, int I, J, the base and the power.  J should be nonnegative.
//
//    Output, int I4_POWER, the value of I^J.
//
{
	int k;
	int value;

	if ( j < 0 )
	{
		if ( i == 1 )
		{
			value = 1;
		}
		else if ( i == 0 )
		{
			ostringstream msg;
			msg << "shared - I4_POWER - Error!\n";
			msg << "  I^J requested, with I = 0 and J negative.\n";
			lowdisc_error(msg.str());
			return 0;
		}
		else
		{
			value = 0;
		}
	}
	else if ( j == 0 )
	{
		if ( i == 0 )
		{
			ostringstream msg;
			msg << "shared - I4_POWER - Error!\n";
			msg << "  I^J requested, with I = 0 and J = 0.\n";
			lowdisc_error(msg.str());
			return 0;
		}
		else
		{
			value = 1;
		}
	}
	else if ( j == 1 )
	{
		value = i;
	}
	else
	{
		value = 1;
		for ( k = 1; k <= j; k++ )
		{
			value = value * i;
		}
	}
	return value;
}
//****************************************************************************80

int prime ( int n )

//****************************************************************************80
//
//  Purpose:
//
//    PRIME returns any of the first PRIME_MAX prime numbers.
//
//  Discussion:
//
//    PRIME_MAX is 1600, and the largest prime stored is 13499.
//
//    Thanks to Bart Vandewoestyne for pointing out a typo, 18 February 2005.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    18 February 2005
//
//  Author:
//
//    John Burkardt
//
//  Reference:
//
//    Milton Abramowitz and Irene Stegun,
//    Handbook of Mathematical Functions,
//    US Department of Commerce, 1964, pages 870-873.
//
//    Daniel Zwillinger,
//    CRC Standard Mathematical Tables and Formulae,
//    30th Edition,
//    CRC Press, 1996, pages 95-98.
//
//  Parameters:
//
//    Input, int N, the index of the desired prime number.
//    In general, is should be true that 0 <= N <= PRIME_MAX.
//    N = -1 returns PRIME_MAX, the index of the largest prime available.
//    N = 0 is legal, returning PRIME = 1.
//
//    Output, int PRIME, the N-th prime.  If N is out of range, PRIME
//    is returned as -1.
//
{
# define PRIME_MAX 1600

	int npvec[PRIME_MAX] = {
		2,    3,    5,    7,   11,   13,   17,   19,   23,   29,
		31,   37,   41,   43,   47,   53,   59,   61,   67,   71,
		73,   79,   83,   89,   97,  101,  103,  107,  109,  113,
		127,  131,  137,  139,  149,  151,  157,  163,  167,  173,
		179,  181,  191,  193,  197,  199,  211,  223,  227,  229,
		233,  239,  241,  251,  257,  263,  269,  271,  277,  281,
		283,  293,  307,  311,  313,  317,  331,  337,  347,  349,
		353,  359,  367,  373,  379,  383,  389,  397,  401,  409,
		419,  421,  431,  433,  439,  443,  449,  457,  461,  463,
		467,  479,  487,  491,  499,  503,  509,  521,  523,  541,
		547,  557,  563,  569,  571,  577,  587,  593,  599,  601,
		607,  613,  617,  619,  631,  641,  643,  647,  653,  659,
		661,  673,  677,  683,  691,  701,  709,  719,  727,  733,
		739,  743,  751,  757,  761,  769,  773,  787,  797,  809,
		811,  821,  823,  827,  829,  839,  853,  857,  859,  863,
		877,  881,  883,  887,  907,  911,  919,  929,  937,  941,
		947,  953,  967,  971,  977,  983,  991,  997, 1009, 1013,
		1019, 1021, 1031, 1033, 1039, 1049, 1051, 1061, 1063, 1069,
		1087, 1091, 1093, 1097, 1103, 1109, 1117, 1123, 1129, 1151,
		1153, 1163, 1171, 1181, 1187, 1193, 1201, 1213, 1217, 1223,
		1229, 1231, 1237, 1249, 1259, 1277, 1279, 1283, 1289, 1291, 
		1297, 1301, 1303, 1307, 1319, 1321, 1327, 1361, 1367, 1373, 
		1381, 1399, 1409, 1423, 1427, 1429, 1433, 1439, 1447, 1451, 
		1453, 1459, 1471, 1481, 1483, 1487, 1489, 1493, 1499, 1511, 
		1523, 1531, 1543, 1549, 1553, 1559, 1567, 1571, 1579, 1583, 
		1597, 1601, 1607, 1609, 1613, 1619, 1621, 1627, 1637, 1657, 
		1663, 1667, 1669, 1693, 1697, 1699, 1709, 1721, 1723, 1733, 
		1741, 1747, 1753, 1759, 1777, 1783, 1787, 1789, 1801, 1811, 
		1823, 1831, 1847, 1861, 1867, 1871, 1873, 1877, 1879, 1889,
		1901, 1907, 1913, 1931, 1933, 1949, 1951, 1973, 1979, 1987,
		1993, 1997, 1999, 2003, 2011, 2017, 2027, 2029, 2039, 2053,
		2063, 2069, 2081, 2083, 2087, 2089, 2099, 2111, 2113, 2129,
		2131, 2137, 2141, 2143, 2153, 2161, 2179, 2203, 2207, 2213,
		2221, 2237, 2239, 2243, 2251, 2267, 2269, 2273, 2281, 2287,
		2293, 2297, 2309, 2311, 2333, 2339, 2341, 2347, 2351, 2357, 
		2371, 2377, 2381, 2383, 2389, 2393, 2399, 2411, 2417, 2423, 
		2437, 2441, 2447, 2459, 2467, 2473, 2477, 2503, 2521, 2531, 
		2539, 2543, 2549, 2551, 2557, 2579, 2591, 2593, 2609, 2617, 
		2621, 2633, 2647, 2657, 2659, 2663, 2671, 2677, 2683, 2687, 
		2689, 2693, 2699, 2707, 2711, 2713, 2719, 2729, 2731, 2741,
		2749, 2753, 2767, 2777, 2789, 2791, 2797, 2801, 2803, 2819, 
		2833, 2837, 2843, 2851, 2857, 2861, 2879, 2887, 2897, 2903, 
		2909, 2917, 2927, 2939, 2953, 2957, 2963, 2969, 2971, 2999, 
		3001, 3011, 3019, 3023, 3037, 3041, 3049, 3061, 3067, 3079, 
		3083, 3089, 3109, 3119, 3121, 3137, 3163, 3167, 3169, 3181, 
		3187, 3191, 3203, 3209, 3217, 3221, 3229, 3251, 3253, 3257, 
		3259, 3271, 3299, 3301, 3307, 3313, 3319, 3323, 3329, 3331, 
		3343, 3347, 3359, 3361, 3371, 3373, 3389, 3391, 3407, 3413, 
		3433, 3449, 3457, 3461, 3463, 3467, 3469, 3491, 3499, 3511, 
		3517, 3527, 3529, 3533, 3539, 3541, 3547, 3557, 3559, 3571,
		3581, 3583, 3593, 3607, 3613, 3617, 3623, 3631, 3637, 3643,
		3659, 3671, 3673, 3677, 3691, 3697, 3701, 3709, 3719, 3727,
		3733, 3739, 3761, 3767, 3769, 3779, 3793, 3797, 3803, 3821,
		3823, 3833, 3847, 3851, 3853, 3863, 3877, 3881, 3889, 3907,
		3911, 3917, 3919, 3923, 3929, 3931, 3943, 3947, 3967, 3989,
		4001, 4003, 4007, 4013, 4019, 4021, 4027, 4049, 4051, 4057,
		4073, 4079, 4091, 4093, 4099, 4111, 4127, 4129, 4133, 4139,
		4153, 4157, 4159, 4177, 4201, 4211, 4217, 4219, 4229, 4231,
		4241, 4243, 4253, 4259, 4261, 4271, 4273, 4283, 4289, 4297,
		4327, 4337, 4339, 4349, 4357, 4363, 4373, 4391, 4397, 4409,
		4421, 4423, 4441, 4447, 4451, 4457, 4463, 4481, 4483, 4493, 
		4507, 4513, 4517, 4519, 4523, 4547, 4549, 4561, 4567, 4583, 
		4591, 4597, 4603, 4621, 4637, 4639, 4643, 4649, 4651, 4657, 
		4663, 4673, 4679, 4691, 4703, 4721, 4723, 4729, 4733, 4751, 
		4759, 4783, 4787, 4789, 4793, 4799, 4801, 4813, 4817, 4831, 
		4861, 4871, 4877, 4889, 4903, 4909, 4919, 4931, 4933, 4937, 
		4943, 4951, 4957, 4967, 4969, 4973, 4987, 4993, 4999, 5003, 
		5009, 5011, 5021, 5023, 5039, 5051, 5059, 5077, 5081, 5087, 
		5099, 5101, 5107, 5113, 5119, 5147, 5153, 5167, 5171, 5179, 
		5189, 5197, 5209, 5227, 5231, 5233, 5237, 5261, 5273, 5279,
		5281, 5297, 5303, 5309, 5323, 5333, 5347, 5351, 5381, 5387,
		5393, 5399, 5407, 5413, 5417, 5419, 5431, 5437, 5441, 5443,
		5449, 5471, 5477, 5479, 5483, 5501, 5503, 5507, 5519, 5521,
		5527, 5531, 5557, 5563, 5569, 5573, 5581, 5591, 5623, 5639,
		5641, 5647, 5651, 5653, 5657, 5659, 5669, 5683, 5689, 5693,
		5701, 5711, 5717, 5737, 5741, 5743, 5749, 5779, 5783, 5791,
		5801, 5807, 5813, 5821, 5827, 5839, 5843, 5849, 5851, 5857,
		5861, 5867, 5869, 5879, 5881, 5897, 5903, 5923, 5927, 5939,
		5953, 5981, 5987, 6007, 6011, 6029, 6037, 6043, 6047, 6053,
		6067, 6073, 6079, 6089, 6091, 6101, 6113, 6121, 6131, 6133,
		6143, 6151, 6163, 6173, 6197, 6199, 6203, 6211, 6217, 6221,
		6229, 6247, 6257, 6263, 6269, 6271, 6277, 6287, 6299, 6301,
		6311, 6317, 6323, 6329, 6337, 6343, 6353, 6359, 6361, 6367, 
		6373, 6379, 6389, 6397, 6421, 6427, 6449, 6451, 6469, 6473, 
		6481, 6491, 6521, 6529, 6547, 6551, 6553, 6563, 6569, 6571, 
		6577, 6581, 6599, 6607, 6619, 6637, 6653, 6659, 6661, 6673, 
		6679, 6689, 6691, 6701, 6703, 6709, 6719, 6733, 6737, 6761, 
		6763, 6779, 6781, 6791, 6793, 6803, 6823, 6827, 6829, 6833, 
		6841, 6857, 6863, 6869, 6871, 6883, 6899, 6907, 6911, 6917, 
		6947, 6949, 6959, 6961, 6967, 6971, 6977, 6983, 6991, 6997,
		7001, 7013, 7019, 7027, 7039, 7043, 7057, 7069, 7079, 7103, 
		7109, 7121, 7127, 7129, 7151, 7159, 7177, 7187, 7193, 7207, 
		7211, 7213, 7219, 7229, 7237, 7243, 7247, 7253, 7283, 7297, 
		7307, 7309, 7321, 7331, 7333, 7349, 7351, 7369, 7393, 7411, 
		7417, 7433, 7451, 7457, 7459, 7477, 7481, 7487, 7489, 7499, 
		7507, 7517, 7523, 7529, 7537, 7541, 7547, 7549, 7559, 7561, 
		7573, 7577, 7583, 7589, 7591, 7603, 7607, 7621, 7639, 7643, 
		7649, 7669, 7673, 7681, 7687, 7691, 7699, 7703, 7717, 7723, 
		7727, 7741, 7753, 7757, 7759, 7789, 7793, 7817, 7823, 7829, 
		7841, 7853, 7867, 7873, 7877, 7879, 7883, 7901, 7907, 7919,
		7927, 7933, 7937, 7949, 7951, 7963, 7993, 8009, 8011, 8017,
		8039, 8053, 8059, 8069, 8081, 8087, 8089, 8093, 8101, 8111,
		8117, 8123, 8147, 8161, 8167, 8171, 8179, 8191, 8209, 8219,
		8221, 8231, 8233, 8237, 8243, 8263, 8269, 8273, 8287, 8291,
		8293, 8297, 8311, 8317, 8329, 8353, 8363, 8369, 8377, 8387,
		8389, 8419, 8423, 8429, 8431, 8443, 8447, 8461, 8467, 8501,
		8513, 8521, 8527, 8537, 8539, 8543, 8563, 8573, 8581, 8597,
		8599, 8609, 8623, 8627, 8629, 8641, 8647, 8663, 8669, 8677,
		8681, 8689, 8693, 8699, 8707, 8713, 8719, 8731, 8737, 8741, 
		8747, 8753, 8761, 8779, 8783, 8803, 8807, 8819, 8821, 8831,
		8837, 8839, 8849, 8861, 8863, 8867, 8887, 8893, 8923, 8929,
		8933, 8941, 8951, 8963, 8969, 8971, 8999, 9001, 9007, 9011,
		9013, 9029, 9041, 9043, 9049, 9059, 9067, 9091, 9103, 9109,
		9127, 9133, 9137, 9151, 9157, 9161, 9173, 9181, 9187, 9199,
		9203, 9209, 9221, 9227, 9239, 9241, 9257, 9277, 9281, 9283,
		9293, 9311, 9319, 9323, 9337, 9341, 9343, 9349, 9371, 9377,
		9391, 9397, 9403, 9413, 9419, 9421, 9431, 9433, 9437, 9439,
		9461, 9463, 9467, 9473, 9479, 9491, 9497, 9511, 9521, 9533,
		9539, 9547, 9551, 9587, 9601, 9613, 9619, 9623, 9629, 9631,
		9643, 9649, 9661, 9677, 9679, 9689, 9697, 9719, 9721, 9733,
		9739, 9743, 9749, 9767, 9769, 9781, 9787, 9791, 9803, 9811,
		9817, 9829, 9833, 9839, 9851, 9857, 9859, 9871, 9883, 9887,
		9901, 9907, 9923, 9929, 9931, 9941, 9949, 9967, 9973,10007,
		10009,10037,10039,10061,10067,10069,10079,10091,10093,10099,
		10103,10111,10133,10139,10141,10151,10159,10163,10169,10177,
		10181,10193,10211,10223,10243,10247,10253,10259,10267,10271,
		10273,10289,10301,10303,10313,10321,10331,10333,10337,10343,
		10357,10369,10391,10399,10427,10429,10433,10453,10457,10459,
		10463,10477,10487,10499,10501,10513,10529,10531,10559,10567,
		10589,10597,10601,10607,10613,10627,10631,10639,10651,10657,
		10663,10667,10687,10691,10709,10711,10723,10729,10733,10739,
		10753,10771,10781,10789,10799,10831,10837,10847,10853,10859,
		10861,10867,10883,10889,10891,10903,10909,10937,10939,10949,
		10957,10973,10979,10987,10993,11003,11027,11047,11057,11059,
		11069,11071,11083,11087,11093,11113,11117,11119,11131,11149,
		11159,11161,11171,11173,11177,11197,11213,11239,11243,11251,
		11257,11261,11273,11279,11287,11299,11311,11317,11321,11329,
		11351,11353,11369,11383,11393,11399,11411,11423,11437,11443,
		11447,11467,11471,11483,11489,11491,11497,11503,11519,11527,
		11549,11551,11579,11587,11593,11597,11617,11621,11633,11657,
		11677,11681,11689,11699,11701,11717,11719,11731,11743,11777,
		11779,11783,11789,11801,11807,11813,11821,11827,11831,11833,
		11839,11863,11867,11887,11897,11903,11909,11923,11927,11933,
		11939,11941,11953,11959,11969,11971,11981,11987,12007,12011,
		12037,12041,12043,12049,12071,12073,12097,12101,12107,12109,
		12113,12119,12143,12149,12157,12161,12163,12197,12203,12211,
		12227,12239,12241,12251,12253,12263,12269,12277,12281,12289,
		12301,12323,12329,12343,12347,12373,12377,12379,12391,12401,
		12409,12413,12421,12433,12437,12451,12457,12473,12479,12487,
		12491,12497,12503,12511,12517,12527,12539,12541,12547,12553,
		12569,12577,12583,12589,12601,12611,12613,12619,12637,12641, 
		12647,12653,12659,12671,12689,12697,12703,12713,12721,12739, 
		12743,12757,12763,12781,12791,12799,12809,12821,12823,12829, 
		12841,12853,12889,12893,12899,12907,12911,12917,12919,12923, 
		12941,12953,12959,12967,12973,12979,12983,13001,13003,13007, 
		13009,13033,13037,13043,13049,13063,13093,13099,13103,13109, 
		13121,13127,13147,13151,13159,13163,13171,13177,13183,13187, 
		13217,13219,13229,13241,13249,13259,13267,13291,13297,13309, 
		13313,13327,13331,13337,13339,13367,13381,13397,13399,13411, 
		13417,13421,13441,13451,13457,13463,13469,13477,13487,13499 };

		if ( n == -1 )
		{
			return PRIME_MAX;
		}
		else if ( n == 0 )
		{
			return 1;
		}
		else if ( n <= PRIME_MAX )
		{
			return npvec[n-1];
		}
		else
		{
			ostringstream msg;
			msg << "shared - PRIME - Error!\n";
			msg << "  Unexpected input value of n = " << n << "\n";
			lowdisc_error(msg.str());
			return 0;
		}

		return 0;
# undef PRIME_MAX
}
//****************************************************************************80
double r8_epsilon ( void )

//****************************************************************************80
//
//  Purpose:
//
//    R8_EPSILON returns the R8 roundoff unit.
//
//  Discussion:
//
//    The roundoff unit is a number R which is a power of 2 with the property
//    that, to the precision of the computer's arithmetic,
//      1 < 1 + R
//    but 
//      1 = ( 1 + R / 2 )
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    01 July 2004
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Output, double R8_EPSILON, the round-off unit.
//
{
	double r;

	r = 1.0;

	while ( 1.0 < ( double ) ( 1.0 + r )  )
	{
		r = r / 2.0;
	}

	return ( 2.0 * r );
}
//****************************************************************************80

void timestamp ( void )

//****************************************************************************80
//
//  Purpose:
//
//    TIMESTAMP prints the current YMDHMS date as a time stamp.
//
//  Example:
//
//    May 31 2001 09:45:54 AM
//
//  Modified:
//
//    02 October 2003
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    None
//
{
# define TIME_SIZE 40

	static char time_buffer[TIME_SIZE];
	const struct tm *tm;
	size_t len;
	time_t now;

	now = time ( NULL );
	tm = localtime ( &now );

	len = strftime ( time_buffer, TIME_SIZE, "%d %B %Y %I:%M:%S %p", tm );

	cout << time_buffer << "\n";

	return;
# undef TIME_SIZE
}
//****************************************************************************80

char *timestring ( void )

//****************************************************************************80
//
//  Purpose:
//
//    TIMESTRING returns the current YMDHMS date as a string.
//
//  Example:
//
//    May 31 2001 09:45:54 AM
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    13 June 2003
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Output, char *TIMESTRING, a string containing the current YMDHMS date.
//
{
# define TIME_SIZE 40

	const struct tm *tm;
	size_t len;
	time_t now;
	char *s;

	now = time ( NULL );
	tm = localtime ( &now );

	s = new char[TIME_SIZE];

	len = strftime ( s, TIME_SIZE, "%d %B %Y %I:%M:%S %p", tm );

	return s;
# undef TIME_SIZE
}

//****************************************************************************80

long long int i8_max ( long long int i1, long long int i2 )

//****************************************************************************80
//
//  Purpose:
//
//    I8_MAX returns the maximum of two I8's.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    12 May 2007
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, long long int I1, I2, two integers to be compared.
//
//    Output, long long int I8_MAX, the larger of I1 and I2.
//
{
	if ( i2 < i1 ) 
	{
		return i1;
	}
	else 
	{
		return i2;
	}

}
//****************************************************************************80

long long int i8_min ( long long int i1, long long int i2 )

//****************************************************************************80
//
//  Purpose:
//
//    I8_MIN returns the smaller of two I8's.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    12 May 2007
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, long long int I1, I2, two integers to be compared.
//
//    Output, long long int I8_MIN, the smaller of I1 and I2.
//
{
	if ( i1 < i2 ) 
	{
		return i1;
	}
	else 
	{
		return i2;
	}

}
//****************************************************************************80

int r8_nint ( double x )

//****************************************************************************80
//
//  Purpose:
//
//    R8_NINT returns the nearest integer to an R8.
//
//  Examples:
//
//        X         R8_NINT
//
//      1.3         1
//      1.4         1
//      1.5         1 or 2
//      1.6         2
//      0.0         0
//     -0.7        -1
//     -1.1        -1
//     -1.6        -2
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    14 November 2006
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, double X, the value.
//
//    Output, int R8_NINT, the nearest integer to X.
//
{
	int value;

	if ( x < 0.0 )
	{
		value = - ( int ) ( r8_abs ( x ) + 0.5 );
	}
	else
	{
		value =   ( int ) ( r8_abs ( x ) + 0.5 );
	}

	return value;
}
//****************************************************************************80

double r8_uniform_01 ( int *seed )

//****************************************************************************80
//
//  Purpose:
//
//    R8_UNIFORM_01 returns a unit pseudorandom R8.
//
//  Discussion:
//
//    This routine implements the recursion
//
//      seed = 16807 * seed mod ( 2**31 - 1 )
//      r8_uniform_01 = seed / ( 2**31 - 1 )
//
//    The integer arithmetic never requires more than 32 bits,
//    including a sign bit.
//
//    If the initial seed is 12345, then the first three computations are
//
//      Input     Output      R8_UNIFORM_01
//      SEED      SEED
//
//         12345   207482415  0.096616
//     207482415  1790989824  0.833995
//    1790989824  2035175616  0.947702
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    11 August 2004
//
//  Author:
//
//    John Burkardt
//
//  Reference:
//
//    Paul Bratley, Bennett Fox, Linus Schrage,
//    A Guide to Simulation,
//    Springer Verlag, pages 201-202, 1983.
//
//    Pierre L'Ecuyer,
//    Random Number Generation,
//    in Handbook of Simulation
//    edited by Jerry Banks,
//    Wiley Interscience, page 95, 1998.
//
//    Bennett Fox,
//    Algorithm 647:
//    Implementation and Relative Efficiency of Quasirandom
//    Sequence Generators,
//    ACM Transactions on Mathematical Software,
//    Volume 12, Number 4, pages 362-376, 1986.
//
//    Peter Lewis, Allen Goodman, James Miller,
//    A Pseudo-Random Number Generator for the System/360,
//    IBM Systems Journal,
//    Volume 8, pages 136-143, 1969.
//
//  Parameters:
//
//    Input/output, int *SEED, the "seed" value.  Normally, this
//    value should not be 0.  On output, SEED has been updated.
//
//    Output, double R8_UNIFORM_01, a new pseudorandom variate, 
//    strictly between 0 and 1.
//
{
	int k;
	double r;

	if ( *seed == 0 )
	{
		ostringstream msg;
		msg << "shared - R8_UNIFORM_01 - Error!\n";
		msg << "  Input value of SEED = 0.\n";
		lowdisc_error(msg.str());
		return 0;
	}

	k = *seed / 127773;

	*seed = 16807 * ( *seed - k * 127773 ) - k * 2836;

	if ( *seed < 0 )
	{
		*seed = *seed + 2147483647;
	}
	//
	//  Although SEED can be represented exactly as a 32 bit integer,
	//  it generally cannot be represented exactly as a 32 bit real number!
	//
	r = ( double ) ( *seed ) * 4.656612875E-10;

	return r;
}
//****************************************************************************80

//****************************************************************************80

float r4_abs ( float x )

//****************************************************************************80
//
//  Purpose:
//
//    R4_ABS returns the absolute value of an R4.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    01 December 2006
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, float X, the quantity whose absolute value is desired.
//
//    Output, float R4_ABS, the absolute value of X.
//
{
	float value;

	if ( 0.0 <= x )
	{
		value = x;
	} 
	else
	{
		value = -x;
	}
	return value;
}
//****************************************************************************80

int r4_nint ( float x )

//****************************************************************************80
//
//  Purpose:
//
//    R4_NINT returns the nearest integer to an R4.
//
//  Example:
//
//        X         R4_NINT
//
//      1.3         1
//      1.4         1
//      1.5         1 or 2
//      1.6         2
//      0.0         0
//     -0.7        -1
//     -1.1        -1
//     -1.6        -2
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    14 November 2006
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, float X, the value.
//
//    Output, int R4_NINT, the nearest integer to X.
//
{
	int value;

	if ( x < 0.0 )
	{
		value = - ( int ) ( r4_abs ( x ) + 0.5 );
	}
	else
	{
		value =   ( int ) ( r4_abs ( x ) + 0.5 );
	}

	return value;
}
//****************************************************************************80

float r4_uniform_01 ( int *seed )

//****************************************************************************80
//
//  Purpose:
//
//    R4_UNIFORM_01 returns a unit pseudorandom R4.
//
//  Discussion:
//
//    This routine implements the recursion
//
//      seed = 16807 * seed mod ( 2**31 - 1 )
//      r4_uniform_01 = seed / ( 2**31 - 1 )
//
//    The integer arithmetic never requires more than 32 bits,
//    including a sign bit.
//
//    If the initial seed is 12345, then the first three computations are
//
//      Input     Output      R4_UNIFORM_01
//      SEED      SEED
//
//         12345   207482415  0.096616
//     207482415  1790989824  0.833995
//    1790989824  2035175616  0.947702
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    16 November 2004
//
//  Author:
//
//    John Burkardt
//
//  Reference:
//
//    Paul Bratley, Bennett Fox, Linus Schrage,
//    A Guide to Simulation,
//    Springer Verlag, pages 201-202, 1983.
//
//    Pierre L'Ecuyer,
//    Random Number Generation,
//    in Handbook of Simulation
//    edited by Jerry Banks,
//    Wiley Interscience, page 95, 1998.
//
//    Bennett Fox,
//    Algorithm 647:
//    Implementation and Relative Efficiency of Quasirandom
//    Sequence Generators,
//    ACM Transactions on Mathematical Software,
//    Volume 12, Number 4, pages 362-376, 1986.
//
//    Peter Lewis, Allen Goodman, James Miller,
//    A Pseudo-Random Number Generator for the System/360,
//    IBM Systems Journal,
//    Volume 8, pages 136-143, 1969.
//
//  Parameters:
//
//    Input/output, int *SEED, the "seed" value.  Normally, this
//    value should not be 0.  On output, SEED has been updated.
//
//    Output, float R4_UNIFORM_01, a new pseudorandom variate, strictly between
//    0 and 1.
//
{
	int k;
	float r;

	if ( *seed == 0 )
	{
		ostringstream msg;
		msg << "shared - R4_UNIFORM_01 - Error!\n";
		msg << "  Input value of SEED = 0.\n";
		lowdisc_error(msg.str());
		return 0;
	}

	k = *seed / 127773;

	*seed = 16807 * ( *seed - k * 127773 ) - k * 2836;

	if ( *seed < 0 )
	{
		*seed = *seed + 2147483647;
	}
	//
	//  Although SEED can be represented exactly as a 32 bit integer,
	//  it generally cannot be represented exactly as a 32 bit real number!
	//
	r = ( float ) ( *seed ) * ( float ) 4.656612875E-10;

	return r;
}

//****************************************************************************80

long long int i8_uniform ( long long int a, long long int b, int *seed )

//****************************************************************************80
//
//  Purpose:
//
//    I8_UNIFORM returns a scaled pseudorandom I8.
//
//  Discussion:
//
//    The pseudorandom number should be uniformly distributed
//    between A and B.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    12 May 2007
//
//  Author:
//
//    John Burkardt
//
//  Reference:
//
//    Paul Bratley, Bennett Fox, Linus Schrage,
//    A Guide to Simulation,
//    Springer Verlag, pages 201-202, 1983.
//
//    Pierre L'Ecuyer,
//    Random Number Generation,
//    in Handbook of Simulation,
//    edited by Jerry Banks,
//    Wiley Interscience, page 95, 1998.
//
//    Bennett Fox,
//    Algorithm 647:
//    Implementation and Relative Efficiency of Quasirandom
//    Sequence Generators,
//    ACM Transactions on Mathematical Software,
//    Volume 12, Number 4, pages 362-376, 1986.
//
//    Peter Lewis, Allen Goodman, James Miller
//    A Pseudo-Random Number Generator for the System/360,
//    IBM Systems Journal,
//    Volume 8, pages 136-143, 1969.
//
//  Parameters:
//
//    Input, long long int A, B, the limits of the interval.
//
//    Input/output, int *SEED, the "seed" value, which should NOT be 0.
//    On output, SEED has been updated.
//
//    Output, long long int I8_UNIFORM, a number between A and B.
//
{
	long long int k;
	double r;
	long long int value;

	if ( *seed == 0 )
	{
		ostringstream msg;
		msg << "shared - I8_UNIFORM - Error!\n";
		msg << "  Input value of SEED = 0.\n";
		lowdisc_error(msg.str());
		return 0;
	}

	k = *seed / 127773;

	*seed = ( int ) ( 16807 * ( *seed - k * 127773 ) - k * 2836) ;

	if ( *seed < 0 )
	{
		*seed = *seed + 2147483647;
	}

	r = ( double ) ( *seed ) * 4.656612875E-10;
	//
	//  Scale R to lie between A-0.5 and B+0.5.
	//
	r = ( 1.0 - r ) * ( ( double ) ( i8_min ( a, b ) ) - 0.5 ) 
		+         r   * ( ( double ) ( i8_max ( a, b ) ) + 0.5 );
	//
	//  Use rounding to convert R to an integer between A and B.
	//
	value = r8_nint ( r );

	value = i8_max ( value, i8_min ( a, b ) );
	value = i8_min ( value, i8_max ( a, b ) );

	return value;
}
//****************************************************************************80

unsigned long long int i8_xor ( unsigned long long int i, 
							   unsigned long long int j )

							   //****************************************************************************80
							   //
							   //  Purpose:
							   //
							   //    I8_XOR calculates the exclusive OR of two I8's.
							   //
							   //  Licensing:
							   //
							   //    This code is distributed under the GNU LGPL license. 
							   //
							   //  Modified:
							   //
							   //    12 May 2007
							   //
							   //  Author:
							   //
							   //   John Burkardt
							   //
							   //  Parameters:
							   //
							   //    Input, unsigned long long int I, J, two values to be exclusive OR-ed.
							   //
							   //    Output, unsigned long long int I8_XOR, the exclusive OR of I and J.
							   //
{
	unsigned long long int i2;
	unsigned long long int j2;
	unsigned long long int k;
	unsigned long long int l;

	k = 0;
	l = 1;

	while ( i != 0 || j != 0 )
	{
		i2 = i / 2;
		j2 = j / 2;

		if ( 
			( ( i == 2 * i2 ) && ( j != 2 * j2 ) ) ||
			( ( i != 2 * i2 ) && ( j == 2 * j2 ) ) )
		{
			k = k + l;
		}

		i = i2;
		j = j2;
		l = 2 * l;
	}

	return k;
}
//****************************************************************************80

int i4_bit_hi1 ( int n )

//****************************************************************************80
//
//  Purpose:
//
//    I4_BIT_HI1 returns the position of the high 1 bit base 2 in an integer.
//
//  Example:
//
//       N    Binary    Hi 1
//    ----    --------  ----
//       0           0     0
//       1           1     1
//       2          10     2
//       3          11     2 
//       4         100     3
//       5         101     3
//       6         110     3
//       7         111     3
//       8        1000     4
//       9        1001     4
//      10        1010     4
//      11        1011     4
//      12        1100     4
//      13        1101     4
//      14        1110     4
//      15        1111     4
//      16       10000     5
//      17       10001     5
//    1023  1111111111    10
//    1024 10000000000    11
//    1025 10000000001    11
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    13 March 2003
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, int N, the integer to be measured.
//    N should be nonnegative.  If N is nonpositive, I4_BIT_HI1
//    will always be 0.
//
//    Output, int I4_BIT_HI1, the location of the high order bit.
//
{
	int bit;

	bit = 0;

	while ( 0 < n )
	{
		bit = bit + 1;
		n = n / 2;
	}

	return bit;
}
//****************************************************************************80

int i4_bit_lo0 ( int n )

//****************************************************************************80
//
//  Purpose:
//
//    I4_BIT_LO0 returns the position of the low 0 bit base 2 in an integer.
//
//  Example:
//
//       N    Binary    Lo 0
//    ----    --------  ----
//       0           0     1
//       1           1     2
//       2          10     1
//       3          11     3 
//       4         100     1
//       5         101     2
//       6         110     1
//       7         111     4
//       8        1000     1
//       9        1001     2
//      10        1010     1
//      11        1011     3
//      12        1100     1
//      13        1101     2
//      14        1110     1
//      15        1111     5
//      16       10000     1
//      17       10001     2
//    1023  1111111111     1
//    1024 10000000000     1
//    1025 10000000001     1
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    13 March 2003
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, int N, the integer to be measured.
//    N should be nonnegative.
//
//    Output, int I4_BIT_LO0, the position of the low 1 bit.
//
{
	int bit;
	int n2;

	bit = 0;

	while ( true )
	{
		bit = bit + 1;
		n2 = n / 2;

		if ( n == 2 * n2 )
		{
			break;
		}

		n = n2;

	}

	return bit;
}

//****************************************************************************80

int i4_uniform ( int a, int b, int *seed )

//****************************************************************************80
//
//  Purpose:
//
//    I4_UNIFORM returns a scaled pseudorandom I4.
//
//  Discussion:
//
//    The pseudorandom number should be uniformly distributed
//    between A and B.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    12 November 2006
//
//  Author:
//
//    John Burkardt
//
//  Reference:
//
//    Paul Bratley, Bennett Fox, Linus Schrage,
//    A Guide to Simulation,
//    Springer Verlag, pages 201-202, 1983.
//
//    Pierre L'Ecuyer,
//    Random Number Generation,
//    in Handbook of Simulation,
//    edited by Jerry Banks,
//    Wiley Interscience, page 95, 1998.
//
//    Bennett Fox,
//    Algorithm 647:
//    Implementation and Relative Efficiency of Quasirandom
//    Sequence Generators,
//    ACM Transactions on Mathematical Software,
//    Volume 12, Number 4, pages 362-376, 1986.
//
//    Peter Lewis, Allen Goodman, James Miller
//    A Pseudo-Random Number Generator for the System/360,
//    IBM Systems Journal,
//    Volume 8, pages 136-143, 1969.
//
//  Parameters:
//
//    Input, int A, B, the limits of the interval.
//
//    Input/output, int *SEED, the "seed" value, which should NOT be 0.
//    On output, SEED has been updated.
//
//    Output, int I4_UNIFORM, a number between A and B.
//
{
	int k;
	float r;
	int value;
	float minab;
	float maxab;

	if ( *seed == 0 )
	{
		ostringstream msg;
		msg << "shared - I4_UNIFORM - Error " << endl;
		msg << "  Input value of SEED = 0" << endl;
		lowdisc_error(msg.str());
		return(0);
	}

	k = *seed / 127773;

	*seed = 16807 * ( *seed - k * 127773 ) - k * 2836;

	if ( *seed < 0 )
	{
		*seed = *seed + 2147483647;
	}

	r = ( float ) ( *seed ) * ( float ) 4.656612875E-10;
	//
	//  Scale R to lie between A-0.5 and B+0.5.
	//
	minab = ( float ) ( i4_min ( a, b ) );
	maxab = ( float ) ( i4_max ( a, b ) );
	r = ( ( float ) 1.0 - r ) * ( minab - ( float ) 0.5 ) + r * ( maxab + ( float ) 0.5 );
	//
	//  Use rounding to convert R to an integer between A and B.
	//
	value = r4_nint ( r );

	value = i4_max ( value, i4_min ( a, b ) );
	value = i4_min ( value, i4_max ( a, b ) );

	return value;
}
//****************************************************************************80

unsigned int i4_xor ( unsigned int i, unsigned int j )

//****************************************************************************80
//
//  Purpose:
//
//    I4_XOR calculates the exclusive OR of two integers.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    16 February 2005
//
//  Author:
//
//   John Burkardt
//
//  Parameters:
//
//    Input, unsigned int I, J, two values whose exclusive OR is needed.
//
//    Output, unsigned int I4_XOR, the exclusive OR of I and J.
//
{
	unsigned int i2;
	unsigned int j2;
	unsigned int k;
	unsigned int l;

	k = 0;
	l = 1;

	while ( i != 0 || j != 0 )
	{
		i2 = i / 2;
		j2 = j / 2;

		if ( 
			( ( i == 2 * i2 ) && ( j != 2 * j2 ) ) ||
			( ( i != 2 * i2 ) && ( j == 2 * j2 ) ) )
		{
			k = k + l;
		}

		i = i2;
		j = j2;
		l = 2 * l;
	}

	return k;
}
//****************************************************************************80

int i8_bit_hi1 ( long long int n )

//****************************************************************************80
//
//  Purpose:
//
//    I8_BIT_HI1 returns the position of the high 1 bit base 2 in an integer.
//
//  Example:
//
//       N    Binary    Hi 1
//    ----    --------  ----
//       0           0     0
//       1           1     1
//       2          10     2
//       3          11     2 
//       4         100     3
//       5         101     3
//       6         110     3
//       7         111     3
//       8        1000     4
//       9        1001     4
//      10        1010     4
//      11        1011     4
//      12        1100     4
//      13        1101     4
//      14        1110     4
//      15        1111     4
//      16       10000     5
//      17       10001     5
//    1023  1111111111    10
//    1024 10000000000    11
//    1025 10000000001    11
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    12 May 2007
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, long long int N, the integer to be measured.
//    N should be nonnegative.  If N is nonpositive, I8_BIT_HI1
//    will always be 0.
//
//    Output, int I8_BIT_HI1, the number of bits base 2.
//
{
	int bit;

	bit = 0;

	while ( 0 < n )
	{
		bit = bit + 1;
		n = n / 2;
	}

	return bit;
}
//****************************************************************************80

int i8_bit_lo0 ( long long int n )

//****************************************************************************80
//
//  Purpose:
//
//    I8_BIT_LO0 returns the position of the low 0 bit base 2 in an integer.
//
//  Example:
//
//       N    Binary    Lo 0
//    ----    --------  ----
//       0           0     1
//       1           1     2
//       2          10     1
//       3          11     3 
//       4         100     1
//       5         101     2
//       6         110     1
//       7         111     4
//       8        1000     1
//       9        1001     2
//      10        1010     1
//      11        1011     3
//      12        1100     1
//      13        1101     2
//      14        1110     1
//      15        1111     5
//      16       10000     1
//      17       10001     2
//    1023  1111111111     1
//    1024 10000000000     1
//    1025 10000000001     1
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license. 
//
//  Modified:
//
//    12 May 2007
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, long long int N, the integer to be measured.
//    N should be nonnegative.
//
//    Output, int I8_BIT_LO0, the position of the low 1 bit.
//
{
	int bit;
	long long int n2;

	bit = 0;

	while ( true )
	{
		bit = bit + 1;
		n2 = n / 2;

		if ( n == 2 * n2 )
		{
			break;
		}

		n = n2;

	}

	return bit;
}
//****************************************************************************80

double r8_abs ( double x )

//****************************************************************************80
//
//  Purpose:
//
//    R8_ABS returns the absolute value of a R8.
//
//  Modified:
//
//    02 April 2005
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, double X, the quantity whose absolute value is desired.
//
//    Output, double R8_ABS, the absolute value of X.
//
{
	double value;

	if ( 0.0 <= x )
	{
		value = x;
	} 
	else
	{
		value = -x;
	}
	return value;
}
//****************************************************************************80

double r8_max ( double x, double y )

//****************************************************************************80
//
//  Purpose:
//
//    R8_MAX returns the maximum of two R8's.
//
//  Modified:
//
//    18 August 2004
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, double X, Y, the quantities to compare.
//
//    Output, double R8_MAX, the maximum of X and Y.
//
{
	double value;

	if ( y < x )
	{
		value = x;
	} 
	else
	{
		value = y;
	}
	return value;
}
//****************************************************************************80

double r8_sign ( double x )

//****************************************************************************80
//
//  Purpose:
//
//    R8_SIGN returns the sign of a R8.
//
//  Modified:
//
//    18 October 2004
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, double X, the number whose sign is desired.
//
//    Output, double R8_SIGN, the sign of X.
//
{
	double value;

	if ( x < 0.0 )
	{
		value = -1.0;
	} 
	else
	{
		value = 1.0;
	}
	return value;
}
//****************************************************************************80
